﻿
Import-Module "$EPiServerPackagingScripts\Get-WebConfig.psm1"
Import-Module "$EPiServerPackagingScripts\Set-WebConfig.psm1"
Import-Module "$EPiServerPackagingScripts\Get-ProtectedModulesPath.psm1"
Import-Module "$EPiServerPackagingScripts\Get-RepositoryPath.psm1"

# moves the procted modules inside modules/_protected folder & writes this info to config files. 
# $projectPath is optional i.e will get from Get-Project cmdlet if not given
Function Move-EPiServerProtectedModules($projectPath)
{	
	$ErrorActionPreference = "Stop"	
	try
	{    
		# fill up projectPath if not given
		if($projectPath -eq $null -or $projectPath -eq "")
		{
			$name = (Get-Project).FullName 
			$projectPath = Split-Path $name
		}

		# quit if no project path
		if($projectPath -eq $null -or $projectPath -eq "")
		{						    
			throw "Project path can't be found. Try to re-run this cmdlet with full path to root directory of the current project."
		}
		
		# get web.config as xml doc
		$elementsToBeLoaded = (,"episerver.framework") # array of elements which we want to expand with web.config
		$config = Get-WebConfig $projectPath $elementsToBeLoaded

		# exit if webConfig is not at root 
		if($config -eq $null -or $config -eq "") 
		{
			throw "WebConfig can't be found. This cmdlet only works when you have a web.config file present at the project's root path."
		}

		# Get the protected modules path
		$protectedPath = Get-ProtectedModulesPath $config $projectPath

		# exit if protectedPath isn't there 
		if($protectedPath -eq $null -or $protectedPath -eq "")
		{
			throw "Current protectedPath can't be found. This cmdlet only works when it can find the current protectedPath location."
		}

		$newProtectedPath = Join-Path $projectPath "modules\_protected"

		# Don't continue if old protected path is pointing to new protected path 
		if(IsParentPath $protectedPath $newProtectedPath)
		{
			throw "The protected modules are already configured to be in /modules/_protected location."
		}

		# set the web.config last change time so it can trigger iis/iis_express to restart app pool
		Set-WebConfigLastWriteTime $projectPath 


		# move the modules folder. This will copy everything from protectedPath to newProtectedPath which is always modules\_protected
		Copy-Folder $protectedPath $newProtectedPath			

		# remove the old modules folder
		if(Test-Path $protectedPath) 
		{
			Remove-Item -Recurse -Force $protectedPath 
		}

		#remove the repository.config from ModuleRepository
		$repositoryPath = Get-RepositoryPath $config $projectPath 
		$repositoryConfig = Join-Path $repositoryPath "repositories.config"

		# it might have been deleted, check it first 
		If(Test-Path $repositoryConfig)
		{
			Remove-Item -Recurse -Force $repositoryConfig
		}

		Update-ConfigEntries $config $projectPath 

		Set-WebConfig $config $projectPath $elementsToBeLoaded
	}
	catch [Exception]
	{
		$ErrorMessage = $_.Exception.Message		
		Show-Message $ErrorMessage

	}	
}

# displays a messagebox with error info
Function Show-Message ($message) 
{
	[System.Windows.Forms.MessageBox]::Show("The cmdlet failed to run due to an error. The error message is: $message") | Out-Null
}

Function Set-WebConfigLastWriteTime($projectPath) 
{
	$webConfigPath = Join-Path $projectPath "web.config"
	if(Test-Path $webConfigPath) 
	{
		(dir $webConfigPath).LastWriteTime = Get-Date
	}
}

# determines if the child path is part of given parent path
Function IsParentPath($parentPath, $childPath) 
{
	$parent = Normalize-Path $parentPath 
	$child = Normalize-Path $childPath 
	return ($parent -eq $child -or $child.Contains($parent)) 
}

Function Normalize-Path($path)
{
	if(([System.IO.Path]::IsPathRooted($path)))
	{
		$fullPath = [System.IO.Path]::GetFullPath($path);
	}
	else
	{
		$fullPath = [System.IO.Path]::GetFullPath((Join-Path (pwd) $path));
	}
	
	return $fullPath.TrimEnd([System.IO.Path]::DirectorySeparatorChar, [System.IO.Path]::AltDirectorySeparatorChar).ToUpperInvariant();     
}


# Update the web.config and episerverframework.config file if found
Function Update-ConfigEntries($config, $projectPath)
{
	# Update the packaging section first 

	$packagingElement = $config.configuration."episerver.packaging"
	if ($packagingElement -ne $null)
	{
		$packagingElement.SetAttribute("protectedPath", "modules\_protected")
	}

	# Update the vpp physical path
	$epiframework = $config.configuration."episerver.framework"

	if($epiframework -ne $null)
	{
		FOREACH($add in $epiframework.virtualPathProviders.ChildNodes)
		{
			if($add.name -eq "ProtectedAddOns")
			{           
				$add.physicalPath = "modules\_protected"
			}
		}
	} 
}

# copies the source path (including subfolders, files etc.) to the destination path
Function Copy-Folder($sourcePath, $destinationPath)
{	

	robocopy $sourcePath $destinationPath /e	  
}

# Warn the user if the protected modules are not configured to standard path (modules\_protected).
Function ExistsProtectedModulePath($projectPath)
{
		# fill up projectPath if not given
		if($projectPath -eq $null -or $projectPath -eq "")
		{
			$name = (Get-Project).FullName 
			$projectPath = Split-Path $name
		}

		# quit if no project path
		if($projectPath -eq $null -or $projectPath -eq "")
		{						    
			return $true
		}
		
		# get web.config as xml doc
		$elementsToBeLoaded = (,"episerver.framework") # array of elements which we want to expand with web.config
		$config = Get-WebConfig $projectPath $elementsToBeLoaded

		# exit if webConfig is not at root 
		if($config -eq $null -or $config -eq "") 
		{
			return $true
		}

		# Get the protected modules path
		$protectedPath = Get-ProtectedModulesPath $config $projectPath
		$newProtectedPath = Join-Path $projectPath "modules\_protected"

		# Checkif old protected path is pointing to new protected path
		if(!(IsParentPath $protectedPath $newProtectedPath))
		{
			return $false
		}

		return $true
}

Function VerifyProtectedModulePath($projectPath)
{
	if(!(ExistsProtectedModulePath $projectPath))
	{
		Write-Warning "The protected modules are not configured to be in /modules/_protected location. You will not be able to install addons from visual studio. You need to run Move-EPiServerProtectedModules cmdlet."
	}
}


Export-ModuleMember -Function Move-EPiServerProtectedModules, ExistsProtectedModulePath, VerifyProtectedModulePath

# SIG # Begin signature block
# MIIXnAYJKoZIhvcNAQcCoIIXjTCCF4kCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUJ880fVDUeGhx/QABiJGOVDpp
# EtigghLKMIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggTQMIIDuKADAgECAhASn/W83LmZkqPf6+aeK2mOMA0GCSqGSIb3DQEBCwUAMH8x
# CzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEfMB0G
# A1UECxMWU3ltYW50ZWMgVHJ1c3QgTmV0d29yazEwMC4GA1UEAxMnU3ltYW50ZWMg
# Q2xhc3MgMyBTSEEyNTYgQ29kZSBTaWduaW5nIENBMB4XDTE2MDExMzAwMDAwMFoX
# DTE5MDQxMzIzNTk1OVowYzELMAkGA1UEBhMCU0UxEjAQBgNVBAgTCVNUT0NLSE9M
# TTESMBAGA1UEBxMJU1RPQ0tIT0xNMRUwEwYDVQQKFAxFUGlTZXJ2ZXIgQUIxFTAT
# BgNVBAMUDEVQaVNlcnZlciBBQjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
# ggEBALWsGwJJX/DKwasEkA9qAsdlsqP8SjVHN7lXwAt2CfBjDI0rN8DO20OfCgos
# Dw1rsSAs1iNNFrB6tdzM+wXZQrHE+bJAYvEXzmZM1kSQfCLz6qIwxx3cRIz8u3Wb
# lH391Dqz03Hf6Ds8N42QKv3m9gQP6g1OIPwlziVkgZ4ANdAP4CfTKPmg0kFqW+az
# WQs+ccYOZEWBi4oPIvPv1uwAbAKIK9fArAtrta7vIdtNf2FZftuL/kAjz980wDFY
# moYR4IGY2eT0FETkoi+dQOhxIbZEl5ziPr5cpiHDWt3J5gueoQCEhiKFg9Uzoquj
# 07IyexmtsjtDsMenkwOSGt2aefMCAwEAAaOCAWIwggFeMAkGA1UdEwQCMAAwDgYD
# VR0PAQH/BAQDAgeAMCsGA1UdHwQkMCIwIKAeoByGGmh0dHA6Ly9zdi5zeW1jYi5j
# b20vc3YuY3JsMGYGA1UdIARfMF0wWwYLYIZIAYb4RQEHFwMwTDAjBggrBgEFBQcC
# ARYXaHR0cHM6Ly9kLnN5bWNiLmNvbS9jcHMwJQYIKwYBBQUHAgIwGQwXaHR0cHM6
# Ly9kLnN5bWNiLmNvbS9ycGEwEwYDVR0lBAwwCgYIKwYBBQUHAwMwVwYIKwYBBQUH
# AQEESzBJMB8GCCsGAQUFBzABhhNodHRwOi8vc3Yuc3ltY2QuY29tMCYGCCsGAQUF
# BzAChhpodHRwOi8vc3Yuc3ltY2IuY29tL3N2LmNydDAfBgNVHSMEGDAWgBSWO1Pw
# eTOXr32D7y4rzMq3hh5yZjAdBgNVHQ4EFgQUlelWRKcMMuDX80+oWbXEPaHUd7sw
# DQYJKoZIhvcNAQELBQADggEBAIaGfEvw4rJgaEDow3Aea6Fg4LGxAtezhs6bjDZi
# h/IJdcWV1nEc/uhZ5XegmRXn3LaP2RL+ZHmjWrQxv4/aK/ZCFxBV0omny3VnIXsY
# UldnW8589S3a83Dtb3cpF+P57M8Z+Fwt+gyvQJYAyDrpMvgMdOotVFWUVVDESXV/
# ttYmhg3MC0ZLuWHREKR9Jrqe9aFjjbGbQlb8jKBOBDPSykjR2nnb5lBgXyfDG9Gf
# zfzz/ed2V95/NSyk2RQD3Wo/IiR/TMABuJEXzsGIMBGSHe6Yz58IxXox4WNyn26o
# 8NklVx6UVsquwXFANU0b4Z/FDTt0cr4PjxNb/Ww/ogKdSBMwggVZMIIEQaADAgEC
# AhA9eNf5dklgsmF99PAeyoYqMA0GCSqGSIb3DQEBCwUAMIHKMQswCQYDVQQGEwJV
# UzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRy
# dXN0IE5ldHdvcmsxOjA4BgNVBAsTMShjKSAyMDA2IFZlcmlTaWduLCBJbmMuIC0g
# Rm9yIGF1dGhvcml6ZWQgdXNlIG9ubHkxRTBDBgNVBAMTPFZlcmlTaWduIENsYXNz
# IDMgUHVibGljIFByaW1hcnkgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHNTAe
# Fw0xMzEyMTAwMDAwMDBaFw0yMzEyMDkyMzU5NTlaMH8xCzAJBgNVBAYTAlVTMR0w
# GwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEfMB0GA1UECxMWU3ltYW50ZWMg
# VHJ1c3QgTmV0d29yazEwMC4GA1UEAxMnU3ltYW50ZWMgQ2xhc3MgMyBTSEEyNTYg
# Q29kZSBTaWduaW5nIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# l4MeABavLLHSCMTXaJNRYB5x9uJHtNtYTSNiarS/WhtR96MNGHdou9g2qy8hUNqe
# 8+dfJ04LwpfICXCTqdpcDU6kDZGgtOwUzpFyVC7Oo9tE6VIbP0E8ykrkqsDoOatT
# zCHQzM9/m+bCzFhqghXuPTbPHMWXBySO8Xu+MS09bty1mUKfS2GVXxxw7hd924vl
# YYl4x2gbrxF4GpiuxFVHU9mzMtahDkZAxZeSitFTp5lbhTVX0+qTYmEgCscwdyQR
# TWKDtrp7aIIx7mXK3/nVjbI13Iwrb2pyXGCEnPIMlF7AVlIASMzT+KV93i/XE+Q4
# qITVRrgThsIbnepaON2b2wIDAQABo4IBgzCCAX8wLwYIKwYBBQUHAQEEIzAhMB8G
# CCsGAQUFBzABhhNodHRwOi8vczIuc3ltY2IuY29tMBIGA1UdEwEB/wQIMAYBAf8C
# AQAwbAYDVR0gBGUwYzBhBgtghkgBhvhFAQcXAzBSMCYGCCsGAQUFBwIBFhpodHRw
# Oi8vd3d3LnN5bWF1dGguY29tL2NwczAoBggrBgEFBQcCAjAcGhpodHRwOi8vd3d3
# LnN5bWF1dGguY29tL3JwYTAwBgNVHR8EKTAnMCWgI6Ahhh9odHRwOi8vczEuc3lt
# Y2IuY29tL3BjYTMtZzUuY3JsMB0GA1UdJQQWMBQGCCsGAQUFBwMCBggrBgEFBQcD
# AzAOBgNVHQ8BAf8EBAMCAQYwKQYDVR0RBCIwIKQeMBwxGjAYBgNVBAMTEVN5bWFu
# dGVjUEtJLTEtNTY3MB0GA1UdDgQWBBSWO1PweTOXr32D7y4rzMq3hh5yZjAfBgNV
# HSMEGDAWgBR/02Wnwt3su/AwCfNDOfoCrzMxMzANBgkqhkiG9w0BAQsFAAOCAQEA
# E4UaHmmpN/egvaSvfh1hU/6djF4MpnUeeBcj3f3sGgNVOftxlcdlWqeOMNJEWmHb
# cG/aIQXCLnO6SfHRk/5dyc1eA+CJnj90Htf3OIup1s+7NS8zWKiSVtHITTuC5nmE
# FvwosLFH8x2iPu6H2aZ/pFalP62ELinefLyoqqM9BAHqupOiDlAiKRdMh+Q6EV/W
# pCWJmwVrL7TJAUwnewusGQUioGAVP9rJ+01Mj/tyZ3f9J5THujUOiEn+jf0or0oS
# vQ2zlwXeRAwV+jYrA9zBUAHxoRFdFOXivSdLVL4rhF4PpsN0BQrvl8OJIrEfd/O9
# zUPU8UypP7WLhK9k8tAUITGCBDwwggQ4AgEBMIGTMH8xCzAJBgNVBAYTAlVTMR0w
# GwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEfMB0GA1UECxMWU3ltYW50ZWMg
# VHJ1c3QgTmV0d29yazEwMC4GA1UEAxMnU3ltYW50ZWMgQ2xhc3MgMyBTSEEyNTYg
# Q29kZSBTaWduaW5nIENBAhASn/W83LmZkqPf6+aeK2mOMAkGBSsOAwIaBQCgcDAQ
# BgorBgEEAYI3AgEMMQIwADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgor
# BgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG9w0BCQQxFgQUJ7PCA7fN
# 3SuJr3Nw1k4C+LYuSREwDQYJKoZIhvcNAQEBBQAEggEAqwCwgG5QaJHkQFH4GmXM
# uc94jJbRp1eFNznGyY1Z3aoI5vwUD6Svcln9W0fmbIg/GQvLEMRujEfYuGmRW2Ch
# oGVvru/e7jLqKV+ySIExsmHBeZqrVUz5NqBM+NmvL8qP/NbQ8hflwFYEyacjsOYL
# +pKuiUDb70M2y7abMAdwPR3VCqRQRcH3PyNrgjQfdeTBfjWcNJsypXIQeu/yjGJW
# E9bSYekn4bGdIc7GjFXmjKDSrQvxxPM2HXJbk5mcT/1TBtQHXvkDaIiL2cGCeFfz
# tL9aaAwYGoCOuv3nJxF0GXc1RI3ciyoKiqSpHpialwWW3d9OqmeV8b2vbyejTTjn
# aKGCAgswggIHBgkqhkiG9w0BCQYxggH4MIIB9AIBATByMF4xCzAJBgNVBAYTAlVT
# MR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEwMC4GA1UEAxMnU3ltYW50
# ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQSAtIEcyAhAOz/Q4yP6/NW4E2GqY
# GxpQMAkGBSsOAwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqG
# SIb3DQEJBTEPFw0xNjA1MTMxMzA4MDJaMCMGCSqGSIb3DQEJBDEWBBQ3nXi6FPWR
# NlKn7DxeItcekwYEKzANBgkqhkiG9w0BAQEFAASCAQAB62KDZnlB3rryPzKhuVwy
# DJDMbSmVSQ5x12zvFUHE5GiDnN53T5aID6dIU/Qpoy2/i6JDNG9IP4Fyzbm/sXkq
# D/IMk4XEI37xwdjmA/x33JcoOje98a10YhJp0ypInhN+wtiRSkujHYhZnEgMC90C
# h8nwzdjgGuUHXoD4SG9z80DYZcL/VVflkc71SawBnfvb51yMwPZflMhrWhUUDliy
# 843aG2VbcNTXmJzvJlOqW69se+WZtfkRJDVErSkFJc+Lj0hMZew2QnANFCkhH6uZ
# Zf6/0jUBoT7urtF54lbptMj3JfZCeCA7IgFJawaeVDYlax3KrsN1tTWv38+yrvga
# SIG # End signature block
